/*
  Generate C code for function scramble.  Main feature of this function is that
  the amount of code generated by compiling it varies according to seed.
  This enables creation of targets where starting addresses of target functions
  vary.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

static void generate(FILE *out, unsigned seed, int bytes) {
    srandom(seed);
    /* Empirically determined that each step requires 12 bytes */
    unsigned nsteps = bytes/14 + random() % 30;
    unsigned i;

    /* Print header */
    fprintf(out, "unsigned scramble(unsigned val)\n");
    fprintf(out, "{\n");
    fprintf(out, "    volatile unsigned a[10];\n");
    fprintf(out, "    unsigned rval = 0;\n");
    fprintf(out, "    unsigned idx;\n");
    fprintf(out, "    for (idx = 0; idx < 10; idx++)\n");
    fprintf(out, "        a[idx] = val + idx * %ld;\n", random() % (1 << 16));


    for (i = 0; i < nsteps; i++)
	fprintf(out, "    a[%ld] *= %ld;\n", random() % 10, random() % (1 << 16));

    /* Print footer */
    
    fprintf(out, "    for (idx = 0; idx < 10; idx++)\n");
    fprintf(out, "        rval += a[idx];\n");
    fprintf(out, "    return rval;\n");    
    fprintf(out, "}\n");
}

void usage(char *name) {
    printf("Usage: %s [-h] [-s VAL] [-o DEST]\n", name);
    printf("    -h     Print help information\n");
    printf("    -s VAL Set initial seed\n");
    printf("    -b BYTES Set minimum number of bytes of code generated\n");
    printf("    -o DEST Specify destination file\n");
    exit(0);
}

int main(int argc, char *argv[]) {
    int seed = 1;
    FILE *out = stdout;
    int opt;
    int bytes = 100;
    while ((opt = getopt(argc, argv, "hs:b:o:")) > 0) {
	switch (opt) {
	case 'h':
	    usage(argv[0]);
	case 's':
	    seed = atoi(optarg) + 1;
	    break;
	case 'o':
	    out = fopen(optarg, "w");
	    if (out == NULL) {
		fprintf(stderr, "Couldn't open '%s'\n", optarg);
		exit(0);
	    }
	    break;
	case 'b':
	    bytes = atoi(optarg);
	    break;
	default:
	    printf("Invalid option '%c'\n", opt);
	    usage(argv[0]);
	}
    }
    generate(out, (unsigned) seed, bytes);
    return 0;
}
